<?php
/*****************************************************************************
 *
 * rawnumbers.php - Gadget for NagVis displaying perfdata as raw numbers
 * 
 * Copyright (c) 2008-2009 by Sascha Runschke (Contact: Sascha.Runschke@gfkl.com)
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

/*****************************************************************************
 *
 * RAWNUMBERS - a NagVis gadget by Sascha Runschke
 *
 * This gadget displays the raw numbers of performance data from nagios.
 * It's purpose was to help display temperature and humidity on large displays,
 * but it is of course possible to display any other perfdata too.
 *
 * The displayed numbers are rendered as png image with ttf script, while
 * the color of the numbers reflect the state of the service:
 * Green for OK, yellow for WARNING, red for CRITICAL.
 * BUT: this gadget only honors HARD states and considers any SOFT state
 * as OK, no matter if it's WARNING or CRITICAL.
 *
 * The unit of measurement ($uom) needs to be supplied by the check plugin
 * for now. While I know that most of those checks are simple snmpgets, which
 * don't supply UOMs, there is no way to implement this right now.
 * As soon as NagVis will implement usersupplied arguments, I will make this
 * configurable.
 * 
 *****************************************************************************/


// Set dummy perfdata for displaying the gadget in nagvis WUI mode
$sDummyPerfdata = "Temperature=99,5C;99,6;99,9;15;100";

require('./gadgets_core.php');

$fontFile = './rawNumbers/luxirr.ttf';

/*******************************************************************************
 * Sanity checking - we need a ttf font and the freetype2 library installed!
 * ****************************************************************************/ 

// If it does not exist - bail out!
if ( ! file_exists($fontFile) ) {
        errorBox("ERROR: FontFile $fontFile not found");
}

if ( ! function_exists('imagettfbbox') ) {
        errorBox('The FreeType2 library is needed by this gadget!');
}

/*******************************************************************************
 * Calculate needed stuff
 * ****************************************************************************/ 

$value = $aPerfdata[0]['value'];
$uom = $aPerfdata[0]['uom'];
$state = $aOpts['state'];
$stateType = $aOpts['stateType'];
$scale = 100;

// gadget scaling will be introduced with nagvis 1.4.3, therefor do some sanity checking to not break <=1.4.2
if ( isset($aOpts['scale']) ) {
	$scale = $aOpts['scale'];
} else {
	$scale = 100;
}

//==================
// scan gadget_opts
//==================

if (isset($aOpts['opts']) && ($aOpts['opts'] != '')){
   preg_match_all ('/(\w+)=(\w+)/',$aOpts['opts'],$matches,PREG_SET_ORDER);
   for ($i = 0; $i < count($matches); $i++){
      if ($matches[$i][1] == 'uom') { $uom = $matches[$i][2]; }
   }
}


$fontSize = 70 * $scale / 100;

// Rotation 0 means no rotation at all
$fontRotation = 0;


// Calculate the size of the imagebox

$imagebox = imagettfbbox($fontSize, $fontRotation, $fontFile, $value . $uom);
$imgwidth = max(array($imagebox[0], $imagebox[2], $imagebox[4], $imagebox[6]))
  - min(array($imagebox[0], $imagebox[2], $imagebox[4], $imagebox[6]));
$imgheight = max(array($imagebox[1], $imagebox[3], $imagebox[5], $imagebox[7]))
  - min(array($imagebox[1], $imagebox[3], $imagebox[5], $imagebox[7]));

  
/*******************************************************************************
 * Create the actual image
 * ****************************************************************************/ 

// We add a few px to the calculated width/height because of our shadow
$img = @imagecreate($imgwidth + 10, $imgheight + 2);

// Define the colors we want to use
$oBackground = imagecolorallocate($img, 255, 255, 255);
$oGreen = imagecolorallocate($img, 0, 255, 0);
$oYellow = imagecolorallocate($img, 255, 255, 0);
$oRed = imagecolorallocate($img, 255, 0, 0);
$oShadow = imagecolorallocate($img, 0x66, 0x66, 0x66);

// White background made transparent
imagefill($img, 0, 0, $oBackground);
imagecolortransparent($img, $oBackground);

// Colorize dependend on state - but only if the state is non-OK && HARD
if ( $aOpts['state'] != 'OK' && $aOpts['stateType'] == 'HARD' ) {
        if ( $aOpts['state'] == 'WARNING' ) {
                $oColor = $oYellow;
        } else {
                if ( $aOpts['state'] == 'CRITICAL' ) {
                        $oColor = $oRed;
                }
        }
} else {
        $oColor = $oGreen;
}

// Now we create the shadow to make the visual experience deeper...
ImageTTFText($img, $fontSize, $fontRotation, 2, $fontSize+2, $oShadow, $fontFile, $value . $uom);

// Write the perfdata value suceeded by the uom into the image
ImageTTFText($img, $fontSize, $fontRotation, 0, $fontSize, $oColor, $fontFile, $value . $uom);

// Output image.
if(function_exists('imageantialias')) {
        imageantialias($img, true);
}

header("Content-type: image/png");
imagepng($img);
imagedestroy($img);
?>
